#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <dav1d/dav1d.h>

int test_one_input(const uint8_t *data, size_t size);

int test_one_input(const uint8_t *data, size_t size)
{
    Dav1dSettings settings = { 0 };
    Dav1dContext * ctx = NULL;
    Dav1dPicture pic;
    const uint8_t *ptr = data;
    int err;

    dav1d_version();

    dav1d_default_settings(&settings);

    settings.n_threads = 2;

    err = dav1d_open(&ctx, &settings);
    if (err < 0) goto end;

    Dav1dData buf;
    uint8_t *p;

    p = dav1d_data_create(&buf, size);
    if (!p) goto cleanup;
    memcpy(p, ptr, size);

    do {
        if ((err = dav1d_send_data(ctx, &buf)) < 0) {
            if (err != DAV1D_ERR(EAGAIN))
                break;
        }
        memset(&pic, 0, sizeof(pic));
        err = dav1d_get_picture(ctx, &pic);
        if (err == 0) {
            dav1d_picture_unref(&pic);
        } else if (err != DAV1D_ERR(EAGAIN)) {
            break;
        }
    } while (buf.sz > 0);

    if (buf.sz > 0)
        dav1d_data_unref(&buf);

    // drain the decoder
    memset(&pic, 0, sizeof(pic));
    err = dav1d_get_picture(ctx, &pic);

cleanup:
    dav1d_close(&ctx);
end:
    return 0;
}

int main(int argc, char **argv)
{
  if(argc != 2) {
    printf("Usage: %s <file>\n", argv[0]);
    return 0;
  }
  
  unsigned char *sample_bytes = NULL;
  uint32_t sample_size = 0;

  FILE *fp = fopen(argv[1], "rb");
  if(!fp) {
    printf("Error opening %s\n", argv[1]);
    return 0;
  }
  fseek(fp, 0, SEEK_END);
  sample_size = (uint32_t)ftell(fp);
  fseek(fp, 0, SEEK_SET);
  sample_bytes = (unsigned char *)malloc(sample_size);
  fread(sample_bytes, 1, sample_size, fp);
  fclose(fp);
  
  test_one_input(sample_bytes, sample_size);

  return 0;
}


