#define _GNU_SOURCE
#include <err.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <sys/prctl.h>
#include <sys/mman.h>
#include <sys/wait.h>

#define SYSCHK(x) ({          \
  typeof(x) __res = (x);      \
  if (__res == (typeof(x))-1) \
    err(1, "SYSCHK(" #x ")"); \
  __res;                      \
})

static char *read_file(char *name, size_t maxlen) {
  char *buf = malloc(maxlen+1);
  if (buf == NULL)
    err(1, "malloc");
  int fd = SYSCHK(open(name, O_RDONLY));
  size_t len = SYSCHK(read(fd, buf, maxlen));
  buf[len] = '\0';
  SYSCHK(close(fd));
  return buf;
}

static void write_file(char *name, char *buf, bool checkerr) {
  int fd = open(name, O_WRONLY);
  if (fd == -1)
    err(1, "unable to open for writing: %s", name);
  ssize_t write_res = write(fd, buf, strlen(buf));
  if (checkerr && SYSCHK(write_res) != strlen(buf))
    errx(1, "write %s", name);
  SYSCHK(close(fd));
}


static char *const mmap_area = (void*)0x3ffff000;
// may need to be satisfiable by the page allocator
// allocation size: 4M
// number of 4K pages: 1024 (2 page tables worth)
// number of page tables, ideally: 3 (for L1) + 2 (for L2) + 2 (for L3, maybe, idk?)
#define MMAP_SIZE (4*1024*1024)

int main(int argc, char **argv) {
  setbuf(stdout, NULL);
  FILE *helper = popen("echo -n \"$(grep ' cgroup2 ' /proc/mounts | cut -d' ' -f2)$(grep ^0:: /proc/self/cgroup|cut -d: -f3)\"", "r");
  if (!helper)
    err(1, "popen");
  char *self_cgroup = NULL;
  size_t self_cgroup_len = 0;
  SYSCHK(getline(&self_cgroup, &self_cgroup_len, helper));
  pclose(helper);
  SYSCHK(chdir(self_cgroup));
  char mypid_str[100];
  sprintf(mypid_str, "%d", getpid());
  write_file("../dbus.service/cgroup.procs", mypid_str, true);
  if (access("memory.max", W_OK)) {
    if (errno == EACCES)
      err(1, "no write access to cgroup - did you launch via systemd?");
    err(1, "unable to write to memory control");
  }
  write_file("memory.swap.max", "0", true);
  write_file("memory.reclaim", "8G", false);

  unsigned long low_bound = 0, high_bound = 1024*1024;
  while (1) {
    unsigned long limit = (low_bound+high_bound)/2;
    char limit_str[100];
    sprintf(limit_str, "%lu", limit);
    write_file("memory.max", limit_str, false);

    pid_t child = SYSCHK(fork());
    if (child == 0) {
      int fd = SYSCHK(open("/dev/bus/usb/001/001", O_RDONLY));

      char cpid_str[100];
      sprintf(cpid_str, "%d", getpid());
      write_file("cgroup.procs", cpid_str, true);

      char *res = mmap(mmap_area, MMAP_SIZE, PROT_READ, MAP_SHARED|MAP_FIXED_NOREPLACE, fd, 0);
      if (res == MAP_FAILED) {
        _exit(42);
      } else {
        _exit(0);
      }
    }
    int wstatus;
    SYSCHK(wait(&wstatus));
    if (WIFEXITED(wstatus) && WEXITSTATUS(wstatus) == 0) {
      printf("limit %lu, exit 0\n", limit);
      high_bound = limit;
    } else {
      printf("limit %lu, not enough\n", limit);
      low_bound = limit;
    }
    if (high_bound - low_bound < 1024)
      return 0;
  }
}
